#
# One_Way_Trip.py
#

# Importion of modules.
import random
import os

import pygame

# Importion of own modules
from source import Timer
from source import Meter_Counter
from source import Background
from source import Map_Generator
from source import Overlay_Shadow
from source import Sewer
from source import Character
from source import helpers
from source import Tile
from source import Crack
from source import Pipe


# Initialize pygame.
pygame.init()


def play_game():
	"""
	"""


	# Load the background sound first.
	pygame.mixer.music.load( os.path.join( "sounds/scary_pad.ogg" ) )
	pygame.mixer.music.play( -1 )


	#################
	# Game properties
	#################

	# General game properties.
	FRAMES_PER_SECOND = 120
	SCREEN_HEIGHT = 220
	SCREEN_WIDTH = 800
	SCREEN_DIMENSIONS = [ SCREEN_WIDTH, SCREEN_HEIGHT ]
	SCREEN_COLOR_DEPTH = 32
	BACKGROUND_COLOR = ( 0, 0, 0 )
	GAME_TITLE = "One Way Trip"
	GAME_CLOCK = pygame.time.Clock()

	# Game specific properties
	

	##################
	# Set up the game.
	##################

	# Set up the display.
	playground = pygame.display.set_mode( SCREEN_DIMENSIONS, 0, SCREEN_COLOR_DEPTH )
	pygame.display.set_caption( GAME_TITLE )

	while True:
		if show_start_screen( playground, GAME_CLOCK, FRAMES_PER_SECOND ):
			break
		
	while True:
		if main( playground, FRAMES_PER_SECOND, GAME_CLOCK, BACKGROUND_COLOR ) == "restart":
			continue
		else:
			break

	pygame.display.quit()
	pygame.quit()

def main( playground, FRAMES_PER_SECOND, GAME_CLOCK, BACKGROUND_COLOR ):
	"""
	The main method sets up and starts the game.
	"""




	# The meter counter.
	meter_counter = Meter_Counter.Meter_Counter()

	# Set up the background
	background = pygame.sprite.Group()
	background.add( Background.Background( [ 0,0 ] ),
					Background.Background( [ 801, 0 ] ) )


	# Set up the pipes
	pipes = pygame.sprite.Group()
	pipes.add( Pipe.Pipe( [ 0, 0 ] ) )

	# Set up the cracks
	cracks = pygame.sprite.Group()
	

	# Set up the sewers and red eyes.
	sewers = pygame.sprite.Group()
	red_eyes = pygame.sprite.Group()
	

	# Set up the start path and the map generator + the tiles group.
	map_generator = Map_Generator.Map_Generator()
	# The following group is going to contain all path sprites.
	path_tiles = pygame.sprite.Group()

	# Add the start tiles to path.
	path_tiles = add_start_tiles( path_tiles, map_generator )
	# And add another path piece
	path_tiles.add( map_generator.get_random_tile() )


	# Add our character to the game.
	character = pygame.sprite.GroupSingle()
	character.add( Character.Character( Character.get_character_animation_images(), 7, [ 155, 120 ] ) )


	# Set up the overlay shadow.
	overlay_shadow = pygame.sprite.GroupSingle()
	overlay_shadow.add( Overlay_Shadow.Overlay_Shadow() )	
	
	###############
	# Run the game.
	###############
	game_is_running = True

	while game_is_running:

		# Get a list of events.
		eventlist = pygame.event.get()


		# Check if the user has quit.
		if user_quit_the_game( eventlist ):
			game_is_running = False
			continue
			

		##########################
		# Update all game objects.
		##########################
		# Update the meter counter
		if meter_counter.update():
			check_meters_run( meter_counter.meters )

		# Update the background.
		for bg in background:
			if bg.update():
				background.add( Background.Background( [ 801, 0 ] ) )


		# Update the pipes
		for pipe in pipes:
			if pipe.update():
				pipes.add( Pipe.Pipe( [ 801, 0 ] ) )
		

		# Update the cracks.
		if random.randint( 1, 100 ) == 50:
			cracks.add( Crack.Crack() )
		cracks.update()

		# Update the sewers.
		Sewer.Sewer.increase_sewer_creation_time_counter()
		if Sewer.Sewer.create_new_sewer_object():
			sewer_obj = Sewer.Sewer()
			red_eyes.add( Sewer.Red_Eyes( 3, sewer_obj.rect ) )
			sewers.add( sewer_obj )
		sewers.update()
		red_eyes.update()

		# Update the path.
		path_tiles.update()
		if map_generator.update():
			path_tiles.add( map_generator.get_random_tile() )

		# Update the overlay shadow.
		overlay_shadow.update()		


		##################################
		# Update our character each frame.
		##################################
		if character.sprite.update( eventlist ):
			game_is_running = False

		# And check if our character has collided with any obstacles.
		if Character.character_has_collided( character, path_tiles ):
			game_is_running = False


		# If the game has ended.
		if not game_is_running:
			# Reset everything

			character.sprite.running_sound.stop()
			
			retry_or_exit = show_game_over_screen( playground, meter_counter.meters, GAME_CLOCK, FRAMES_PER_SECOND )

			# Reset everything
			character = None
			path_tiles = None
			meter_counter = None
			background = None
			map_generator = None
			overlay_shadow = None
			sewers = None
			cracks = None
			
			Map_Generator.Map_Generator.unique_tiles = 2

			return retry_or_exit

		#######################
		# Draw all game objects
		#######################

		# Fill the background with a black color.
		playground.fill( BACKGROUND_COLOR )

		# Draw the background.
		background.draw( playground )

		# Draw the pipes
		pipes.draw( playground )

		# Draw the cracks
		cracks.draw( playground )

		# Draw the sewers
		sewers.draw( playground )

		# Draw the path.
		path_tiles.draw( playground )

		# Draw our character.
		character.draw( playground )

		# Draw the overlay shadow.
		overlay_shadow.draw( playground )

		# Draw the red eyes.
		red_eyes.draw( playground )		
		
		# Update the entire display.
		GAME_CLOCK.tick( FRAMES_PER_SECOND )
		pygame.display.flip()

	
def user_quit_the_game( eventlist ):
	"""
	"""

	for event in eventlist:
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE:
				return True
	return False


def check_meters_run( meters_run ):
	"""
	This method checks how many meters have been run and whether the level should be increased or not.

	Parameters
		meters_run		Unsigned integer
			The amount of meters our character has run.
		meters_to_run_lvl	Dictionary{ [ unsigned integer meters, unsigned integer level ] [,...] ] }

	Return value
		
	"""

	if meters_run == 100:
		Map_Generator.Map_Generator.unique_tiles += 1
	elif meters_run == 250:
		Map_Generator.Map_Generator.unique_tiles += 2
	#elif meters_run == 425:
	#	Map_Generator.Map_Generator.unique_tiles += 1
	#elif meters_run == 600:
	#	Map_Generator.Map_Generator.unique_tiles += 1
	#elif meters_run == 850:
	#	Map_Generator.Map_Generator.unique_tiles += 1			
						


def update_level( current_level ):
	"""
	This function update the level.

	Parameters
		current_level	Unsigned integer
			An integer representing the current level.

	Return value
		current_level	Unsigned integer
			The updated level.
	"""

	current_level += 1

	return current_level
	

def add_start_tiles( path_tiles, map_generator ):
	"""
	This function adds the start path tiles to 'path_tiles'.  This means that in the beginning,
	the player is standing on a normal path.

	Parameters
		path_tiles		pygame.sprite.Group() object.
			Holds the tiles for the path.

		map_generator	Map_Generator.Map_Generator oject
			A map generator that generates the path tiles.

	Return value
		path_tiles		pygame.sprite.Group() object.
	
	"""

	path_tile_image_width = map_generator.tiles[1].get_rect().width
	paths_to_create = ( 800 / path_tile_image_width ) + 1

	for x in range( paths_to_create ):
		left_top_coord = [ x * path_tile_image_width, 200 ]
		path_tiles.add( Tile.Tile( map_generator.tiles[1], 1, left_top_coord ) )

	return path_tiles


def show_start_screen( playground, clock, fps ):
	"""
	"""

	# The background.
	start_screen = pygame.sprite.Sprite()
	start_screen.image = helpers.load_image( "title_Screen.png" )
	start_screen.rect = start_screen.image.get_rect( topleft = ( 0, 0 ) )

	running = True
	while running:

		# Check if the player wants to start the game by checking for any keypress.
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.KEYDOWN:
				return True

		blit_sprites( playground, [ start_screen ] )
		pygame.display.flip()

		clock.tick( fps )

def show_game_over_screen( playground, meters_run, clock, fps ):
	"""
	"""

	# Set up the sprites to show.

	# The background.
	game_over_background = pygame.sprite.Sprite()
	game_over_background.image = helpers.load_image( "retry_back.png" )
	game_over_background.rect = game_over_background.image.get_rect( center = ( 400, 103 ) )

	# The 'You have made' message.
	you_have_made_image = pygame.sprite.Sprite()
	you_have_made_image.image = helpers.load_image( "you_have_made.png" )
	you_have_made_image.rect = you_have_made_image.image.get_rect( center = ( 400, 60 ) )
	
	# The meters image.
	font1 = pygame.font.SysFont( "SysFont", 30, False, False )

	meters_run_image = pygame.sprite.Sprite()
	meters_run_image.image = font1.render( str( meters_run ) + " meters", True, ( 100, 100, 100 ) )
	meters_run_image.rect = meters_run_image.image.get_rect( center = ( 400, 90 ) )
	
	# The retry button
	retry_image = pygame.sprite.Sprite()
	retry_image.image = helpers.load_image( "retry.png" )
	retry_image.rect = retry_image.image.get_rect( center = ( 370, 130 ) )
	
	# The exit button
	exit_button = pygame.sprite.Sprite()
	exit_button.image = helpers.load_image( "quit.png" )
	exit_button.rect = exit_button.image.get_rect( center = ( 430, 130 ) )

	running = True
	while running:

		# Check if the player wants to retry or quit.
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.MOUSEBUTTONDOWN:
				# Which mousebutton has been pressed ?
				pressed_mouse_buttons = pygame.mouse.get_pressed()
				# Is our mousebutton over the retry button ?
				if mousepos_inside_rect( pygame.mouse.get_pos(), retry_image.rect ) and pressed_mouse_buttons[0] == True:
					# Restart the game.
					return "restart"
				elif mousepos_inside_rect( pygame.mouse.get_pos(), exit_button.rect ) and pressed_mouse_buttons[0] == True:
					return "exit"
				
		blit_sprites( playground, [ game_over_background, you_have_made_image, retry_image, exit_button, meters_run_image  ] )
		pygame.display.flip()

		clock.tick( fps )
		

def mousepos_inside_rect( mousepos, rect ):
	"""
	"""

	if mousepos[0] > rect.left and mousepos[0] < rect.right and mousepos[1] > rect.top and mousepos[1] < rect.bottom:
		return True
	else:
		return False

def blit_sprites( playground, sprite_list ):
	"""
	"""

	for sprite in sprite_list:
		playground.blit( sprite.image, sprite.rect )

		

	
######################################################

# Run the game.
play_game()
