# Animation.py

# Importion of modules as always.
import pygame

import helpers

# Initialize pygame
pygame.init()

class Animation( object ):
	"""
	The animation class.  This class is used for animations.
	"""

	def __init__( self, images, update_interval, reverse = False ):
		"""
		Initialize the animation object.

		Parameters
			images				pygame.Surface list.
				This list contains all the pictures in the correct order.
			update_interval		Integer
				Defines the interval between image switches. In frames per second.
			reverse				Boolean
				If set to true, the list of images is walked forth and back
		"""

		# First, let us store the parameters
		self.images = images
		self.update_interval = update_interval # in fps
		self.reverse = reverse

		# Store the amount of images.
		self.n_images = len( self.images )
		self.current_image = 0

		# Set up the image.
		self.image = self.images[0]

		# Set up the interval counter.
		self.interval_counter = 0
		

	def update( self ):
		"""
		Updates the animation.
		"""

		if self.need_to_change_image():
			self.change_image()
			self.reset_interval_counter()

			return True
		else:
			self.increase_interval_counter()

		
	def increase_interval_counter( self ):
		"""
		This method increases the value of the interval counter by 1.
		"""

		self.interval_counter += 1


	def need_to_change_image( self ):
		"""
		This method returns 'True' if the image has to be changed.  Otherwise, 'False' is returned.

		Return value
			Boolean
		"""

		if self.interval_counter == self.update_interval:
			return True
		else:
			return False

	def reset_interval_counter( self ):
		"""
		This method resets the interval counter.
		"""

		self.interval_counter = 0


	def change_image( self ):
		"""
		This method changes the image.
		"""

		if self.current_image + 1 == self.n_images:
			# The last image was displayed.  Either go backwards, or start from the beginning again.
			self.current_image = 0
		else:
			self.current_image += 1

		self.image = self.images[ self.current_image ]
