# Importion of modules
import pygame
import random

import Tile
import helpers


class Map_Generator( object ):
	"""
	The map generator creates the map or rather, the path of the game.  The path can, up until now, only be generated randomly.  It is also important to note, that the player can max. jump two obstacles at a time, so there may not be more than 2 obstacles following each other.
	"""


	unique_tiles = 2
	
	def __init__( self ):
		"""
		This method initializes the map generator object.
		"""

		# Properties.
		# This is a list of all the possible tiles for the map.  It includes the path and some of the obstacles.  Other obstacles, like the larvae, that can also move, will be included later in their own class.
		self.tiles = { 	1: helpers.load_image( "ground.png" ),
						2: helpers.load_image( "hole.png" ),
						3: helpers.load_image( "ground_peak.png" ),
						4: helpers.load_image( "crates_1.png" ),
						#5: helpers.load_image( "crates_2.png" ),
						#6: helpers.load_image( "crates_3.png" ),
						#7: helpers.load_image( "crates_4.png" )
					};

		# Defines a counter that counts up until the value of self.new_map_tile_at has been reached.
		self.new_map_tile_counter = 0
		# This property defines the time of _when_ to create a new tile for our random map.		
		self.new_map_tile_at = 0


		# Game logic.
		self.max_continuous_obstacles = 2
		self.continuous_obstacles = 0
		

	def get_random_tile( self ):
		"""
		Returns a random tile for the map.
		Could be an obstacle, or simply a normal tile.
		"""

		# Will our next tile be an obstacle or just a normal path ?
		random_number = random.randint( 1, Map_Generator.unique_tiles )

		# Check whether an obstacle will be generated.
		if self.is_obstacle( random_number ):
			# If two obstacles have been generated one after another already, then no obstacle may be generated.
			if self.continuous_obstacles == self.max_continuous_obstacles:
				random_number = 1
				self.continuous_obstacles = 0
			else:
				self.continuous_obstacles += 1
		else:
			self.continuous_obstacles = 0

		# Get and store the image for the path-tile.
		image = self.tiles[ random_number ]

		# We need to store the width of the image that will be generated, so that we know, when to create a new tile for our random path.
		self.get_img_width( image )

		# Finally, generate and return our new tile.
		return Tile.Tile( self.tiles[random_number], random_number )


	def get_img_width( self, rand_img ):
		"""
		This method returns the width of an image in pixels.

		Parameters
			rand_img 	pygame.surface object.

		Return value
			Integer
				The width of the image in pixels.
		"""

		self.new_map_tile_at = rand_img.get_rect().width / 3 - 1
		

	def update( self ):
		"""
		This method updates the map generator.  This is vital, as we need to continuously create new path elements / path tiles and add them to the game.
		"""

		# Check whether a new tile has to be created and added to the path.
		if self.new_map_tile_counter == self.new_map_tile_at:
			# Add a new tile.
			self.new_map_tile_counter = 0
			return  True
			
		else:
			# Don't add a new tile.
			self.new_map_tile_counter += 1
			return False

	def is_obstacle( self, number ):
		"""
		This method returns 'True' if number is not 1.  Otherwise, 'False' is returned.

		Parameters
			number	Integer
				The integer defines whether the next tile to create is an obstacle or not.

		Return value
			Boolean
		"""

		if number is not 1:
			return True
		return False
