# Tile.py

# Importion of modules
import pygame

class Tile( pygame.sprite.Sprite ):
	"""
	Creates a new tile for the One way path.

	A tile can be one of the following:
		Normal Path
			No obstacle.
		Pik / Spikes
			An obstacle.
		Larvae 1 and 2
			Also an obstacle.

	If the player touches one of the obstacles, he has lost.
	"""

	def __init__( self, image, type_of_tile, opt_left_top = None ):
		"""
		This method initializes a newly created tile.

		Parameters
			image			pygame.surface object.
				The image of the tile.
			top_left_pos	Integer array [ top, left ] position of the image.
				The topleft position of the image.  Defines where the image should be displayed.
			type_of_tile	Integer
				An integer defining what kind of tile it is.  Is it an obstacle ?  Is it a normal path ?
			opt_left_top	Integer array
				An integer array defining the left top coords of where to put the rect.  Optional.
				
		"""
		pygame.sprite.Sprite.__init__( self )

		# Store the passed parameters | properties.
		self.image = image
		self.type_of_tile = type_of_tile

		# Properties.
		self.rect = self.image.get_rect()

		self.is_obstacle = False

		if self.type_of_tile == 1:
			# Normal path.
			self.top_left_pos = [ 200, 800 ]
		elif self.type_of_tile == 2:
			# a hole
			self.top_left_pos = [ 174, 800 ]
			self.is_obstacle = True
		elif self.type_of_tile == 3:
			# A spikes
			self.top_left_pos = [ 174, 800 ]
			self.is_obstacle = True
		elif self.type_of_tile == 4:
			# Crates 1
			self.top_left_pos = [ 167, 800 ]
			self.is_obstacle = True	
		elif self.type_of_tile == 5:
			# Crates 2
			self.top_left_pos = [ 134, 800 ]
			self.is_obstacle = True	
		elif self.type_of_tile == 6:
			# Crates 3
			self.top_left_pos = [ 134, 800 ]
			self.is_obstacle = True	
		elif self.type_of_tile == 7:
			# Crates 4
			self.top_left_pos = [ 101, 800 ]
			self.is_obstacle = True	

									
		self.rect.top = self.top_left_pos[0]
		self.rect.left = self.top_left_pos[1]

		if opt_left_top is not None:
			self.rect.left = opt_left_top[0]		
			self.rect.top = opt_left_top[1]


	def update( self ):
		"""
		"""
		self.rect.move_ip( -3, 0 )
					
		if self.rect.right <= 0:
			self.kill()
